;===================================================================;
;                                                                   ;
; Hangman                                                           ;
; Version 1.0.1                                                     ;
;                                                                   ;
; Copyright (c)2007-2012 James Vernon                               ;
; Written and Programmed by James Vernon <jamesv82@love.com.au>     ;
; http://www.jvti.org                                               ;
;                                                                   ;
; You are allowed to use parts of this code as long as you          ;
; give credit where it is due.                                      ;
;                                                                   ;
;===================================================================;

#include "hangman.h"                            ; Header info

#include "vars.asm"                             ; Variables

;================================================
; Start of Program
;================================================
start:
#ifdef TI83
        res     statsvalid,(iy+statflags)
#endif
#ifdef TI86
        call    _flushallmenus
        call    _runindicoff
        call    _clrscrnfull
        call    _ram_page_1
#endif
#ifndef TI86
        ld      hl,gbuf+768
        ld      de,systemBackup
        ld      bc,256
        ldir
#endif
        set     textwrite,(iy+sgrflags)         ; Write text to gbuf

startVAT:
#ifndef TI86
        ld      hl,(vat)                        ; HL => Start of VAT
#else
        ld      hl,$BFFF                        ; HL => Start of VAT
#endif
        ld      (vatPtr),hl                     ; Save it
        xor     a
        ld      (numLibs),a                     ; Reset number of library counters
nextLib:
        ld      hl,(vatPtr)                     ; Get pointer to next VAT entry
libSearch:
        ld      ix,searchString                 ; IX => Search string
        call    ionDetect                       ; Search for next valid file in VAT
        jp      z,libFound
libNotFound:                                    ; If execution comes to here, we have search to the end of the VAT
        ld      a,(numLibs)                     ; A = Number of libraries we found
        or      a
        jr      nz,startVAT                     ; If we found some, return to start of VAT to recycle through libraries
        bcall(_grbufclr)
        ld      hl,strNoLibs
#ifndef TI86
        ld      de,28*256+14
#else
        ld      de,28*256+31
#endif
        call    vPuts                           ; Display no libraries message
        call    ionFastCopy
        call    waitKey
        jp      quit                            ; Quit

menuScreen:
        ld      hl,titlePicData                 ; HL => Start of compressed data
        ld      de,videoBuffer                  ; DE => Destination for uncompressed data
        ld      ix,gbuf                         ; IX => 1024 bytes free mem to be used by huffExtr
        call    huffExtr                        ; Extract game background to gbuf

        bcall(_grbufclr)

        ld      hl,videoBuffer
        ld      de,gbuf
#ifndef TI86
        ld      bc,192
#else
        ld      bc,256
#endif
        ldir                                    ; Load uncompressed data to gbuf

        ld      hl,strLib
#ifndef TI86
        ld      de,23*256+1
#else
        ld      de,23*256+17
#endif
        call    vPuts                           ; Disp "LIBRARY:"
        ld      hl,strCategories
#ifndef TI86
        ld      de,37*256+16
#else
        ld      de,37*256+32
#endif
        call    vPuts                           ; Disp "CATEGORIES:"
        ld      hl,strQuestions
#ifndef TI86
        ld      de,43*256+16
#else
        ld      de,43*256+32
#endif
        call    vPuts                           ; Disp "QUESTIONS:"
        ld      hl,strHiScore
#ifndef TI86
        ld      de,49*256+16
#else
        ld      de,49*256+32
#endif
        call    vPuts                           ; Disp "HI-SCORE:"
        ld      hl,strJV
#ifndef TI86
        ld      de,14*256+21
#else
        ld      de,14*256+37
#endif
        call    vPuts                           ; Disp "by James Vernon"
        ld      hl,strVersion
#ifndef TI86
        ld      de,57*256+75
#else
        ld      de,57*256+107
#endif
        call    vPuts                           ; Disp version number

        ld      hl,(libNamePtr)
#ifndef TI86
        ld      de,29*256+4
#else
        ld      de,29*256+20
#endif
        call    vPuts                           ; Disp library name
#ifndef TI86
        ld      de,37*256+61
#else
        ld      de,37*256+77
#endif
        ld      hl,(numCategories)
        ld      h,0
        call    setShowHL                       ; Disp number of categories
#ifndef TI86
        ld      de,43*256+61
#else
        ld      de,43*256+77
#endif
        ld      hl,(numQuestions)
        ld      h,0
        call    setShowHL                       ; Disp number of questions
#ifndef TI86
        ld      de,49*256+61
#else
        ld      de,49*256+77
#endif
        ld      hl,(hiScore)
        call    setShowHL                       ; Disp hi score

        call    ionFastCopy
        call    waitKey
        cp      GK_ALPHA                        ; Was [ALPHA] pressed?
        jp      z,nextLib                       ; If so, find next library
        cp      GK_2ND                          ; Was [2nd] pressed?
        jr      z,newGame                       ; If so, start new game with this library
#ifndef TI86
        cp      GK_MODE                         ; Was [MODE] pressed? (83/83+)
#else
        cp      GK_EXIT                         ; Was [EXIT] pressed? (86)
#endif
        jp      nz,menuScreen                   ; If not, no other key presses to check, start menu loop again
        jp      quit                            ; Otherwise, it was pressed, so quit

libFound:
        ld      (vatPtr),de                     ; Save where in VAT we're up to
        ld      a,(hl)                          ; A = Library version
        or      a
        jp      nz,libNotFound                  ; If higher than version 0, not compatible with this version of the game
        inc     hl
        ld      a,(hl)
        ld      (numCategories),a               ; Save number of categories
        ld      b,a                             ; B = Number of categories (needed later)
        inc     hl
        ld      a,(hl)
        dec     a                               ; A = Real number of questions (compression bug fix??)
        ld      (numQuestions),a                ; Save number of questions
        inc     hl
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        ld      (hiScore),de                    ; Save current HiScore
        inc     hl
        ld      (libNamePtr),hl                 ; Save pointer to library name
        inc     b                               ; B = Number of categories (from earlier) + 1
findLibData:
        call    nextString
        djnz    findLibData                     ; Skip B strings to get to start of data
        ld      (dataPtr),hl                    ; Save pointer to start of compressed data
        ld      hl,numLibs
        inc     (hl)                            ; Increase number of Libraries counter
        jp      menuScreen

newGame:
        xor     a                               ; A = 0
        ld      (score),a                       ; Reset some vars for the new game
        ld      (score+1),a
        ld      (level),a
newQuestion:
        ld      a,(numQuestions)
        ld      b,a
        call    random                          ; Get a random number (0 <= A < numQuestions)
        ld      b,a                             ; B = Question number to load
        ld      hl,(dataPtr)                    ; HL => Start of compressed data
        ld      de,question                     ; DE => Where to load question
        ld      ix,videoBuffer                  ; IX => 1024 memory buffer for huffExtr
;        call    hangsrleDecode                  ; Decode question
        call    huffExtr

#include "maingame.asm"                         ; Main game loop

wonGame:
        pop     hl
        pop     hl
        pop     hl                              ; Clear stack
lostGame:
        call    invertScreen
        call    ionFastCopy                     ; Show completed question

; Calculate score addition
        ld      a,(charsWrong)
        add     a,a                             ; A = offset in scoreTable
        ld      hl,scoreTable
        ld      e,a
        ld      d,0                             ; DE = offset in scoreTable
        add     hl,de                           ; HL => entry in scoreTable
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = score to add
        ld      hl,(score)
        add     hl,de                           ; HL = new score
        ld      (score),hl

        call    wait2nd                         ; Wait for [2nd] to be pressed

        ld      hl,level
        inc     (hl)                            ; Increment level counter
        ld      a,(hl)
        cp      10                              ; Check if 10 questions have been completed
        jp      nz,newQuestion                  ; If not, do another question!
        ld      hl,strWon                       ; HL => String to display at endGame

endGame:
        push    hl
        bcall(_grbufclr)                        ; Clear gbuf

        pop     hl                              ; HL => String to display
#ifndef TI86
        ld      de,15*256+34
#else
        ld      de,15*256+49
#endif
        call    vPuts                           ; Disp either "YOU WON!" or "YOU LOST!"

        ld      hl,strScore
#ifndef TI86
        ld      de,26*256+16
#else
        ld      de,26*256+32
#endif
        call    vPuts                           ; Display "YOUR SCORE IS:"

#ifndef TI86
        ld      de,26*256+61
#else
        ld      de,26*256+77
#endif
        ld      hl,(score)
        call    setShowHL                       ; Disp score

        ld      de,(score)
        ld      hl,(hiScore)
        bcall(_cphlde)                          ; Compare players' score to the current high score
        jr      nc,noNewHiScore

        ld      hl,strNewHi
#ifndef TI86
        ld      de,45*256+7
#else
        ld      de,45*256+21
#endif
        call    vPuts                           ; Display "YOU GOT A NEW HIGH SCORE!"

; Save the new high score
        ld      de,(score)
        ld      (hiScore),de                    ; Save to our local variable
#ifdef TI86
        push    de
        ld      hl,(realFilePtr)
        ld      a,(realFilePtr+2)
        ld      de,12
        add     hl,de
        adc     a,0                             ; AHL => High score in external level file
        pop     de                              ; DE = New high score
        call    _set_word_ahl                   ; Save high score for TI-86
        call    _ram_page_1
#else
        ld      hl,(libNamePtr)                 ; HL => Libary name in external level file (high score is located in the preceding 2 bytes)
        dec     hl
        ld      (hl),d
        dec     hl
        ld      (hl),e                          ; Save high score for TI-83/83+
#endif

noNewHiScore:
        call    ionFastCopy                     ; Display end of game screen
        call    wait2nd

        jp      menuScreen

quit:
        res     textwrite,(iy+sgrflags)
#ifndef TI86
        ld      hl,systemBackup
        ld      de,gbuf+768
        ld      bc,256
        ldir
#endif
#ifdef TI86
        call    _clrscrnfull
        call    _homeup
        set     graphdraw,(iy+graphflags)
#endif
#ifdef TI83
        set     statsvalid,(iy+statflags)
#endif
        ret

;================================================
; Include Files (Code)
;================================================
#include "huffextr.asm"                         ; Huffman extractor
#include "misc.asm"                             ; Misc routines
#include "sprite.asm"                           ; Sprite routines

;================================================
; Include Files (Data)
;================================================
#include "gamespr.asm"                          ; In-game sprites
#ifndef TI86
#include "gpic12.asm"                           ; Game screen picture data for TI-83(+)
#include "title12.asm"                          ; Menu title picture data for TI-83(+)
#else
#include "gpic16.asm"                           ; Game screen picture data for TI-86
#include "title16.asm"                          ; Menu title picture data for TI-86
#endif

searchString:
.db     "HANGLIB",0

strNoLibs:
.db     "NO LIBRARIES FOUND!",0

strLib:
.db     "LIBRARY:",0

strCategories:
.db     "CATEGORIES:",0

strQuestions:
.db     "QUESTIONS:",0

strHiScore:
.db     "HI-SCORE:",0

strJV:
.db     "by James Vernon",0

strVersion:
.db     "v",verMajor,".",verMinor,".",verRevision,0

strLost:
.db     "YOU LOST!",0

strWon:
.db     "YOU WON!",0

strScore:
.db     "Your score is:",0

strNewHi:
.db     "YOU GOT A NEW HIGH SCORE!",0

scoreTable:
.dw     1000,700,500,200,50,0

letterTable:                                    ; Letter table for getLetter routine
#ifndef TI86
.db     ".........."
.db     "WRMH....VQ"
.db     "LG...ZUPKF"
.db     "C..YTOJEB."
.db     ".XSNIDA..."
.db     "......"
#else
.db     ".........X"
.db     "TOJE...WSN"
.db     "ID..ZVRMHC"
.db     "..YUQLGB.."
.db     "..PKFA...."
.db     "......"
#endif

questionLetterCoords:
#ifndef TI86
.db     35,20, 35,24, 35,28, 35,32, 35,36, 35,40, 35,44, 35,48, 35,52, 35,56, 35,60, 35,64, 35,68, 35,72
.db     41,20, 41,24, 41,28, 41,32, 41,36, 41,40, 41,44, 41,48, 41,52, 41,56, 41,60, 41,64, 41,68, 41,72
.db     47,20, 47,24, 47,28, 47,32, 47,36, 47,40, 47,44, 47,48, 47,52, 47,56, 47,60, 47,64, 47,68, 47,72
.db     53,20, 53,24, 53,28, 53,32, 53,36, 53,40, 53,44, 53,48, 53,52, 53,56, 53,60, 53,64, 53,68, 53,72
#else
.db     35,36, 35,40, 35,44, 35,48, 35,52, 35,56, 35,60, 35,64, 35,68, 35,72, 35,76, 35,80, 35,84, 35,88
.db     41,36, 41,40, 41,44, 41,48, 41,52, 41,56, 41,60, 41,64, 41,68, 41,72, 41,76, 41,80, 41,84, 41,88
.db     47,36, 47,40, 47,44, 47,48, 47,52, 47,56, 47,60, 47,64, 47,68, 47,72, 47,76, 47,80, 47,84, 47,88
.db     53,36, 53,40, 53,44, 53,48, 53,52, 53,56, 53,60, 53,64, 53,68, 53,72, 53,76, 53,80, 53,84, 53,88
#endif

usedLetterCoords:
#ifndef TI86
.db     06,09, 06,13, 06,17, 06,21, 06,25, 06,29, 06,33, 06,37, 06,41, 06,45, 06,49, 06,53, 06,57
.db     12,09, 12,13, 12,17, 12,21, 12,25, 12,29, 12,33, 12,37, 12,41, 12,45, 12,49, 12,53, 12,57
#else
.db     06,25, 06,29, 06,33, 06,37, 06,41, 06,45, 06,49, 06,53, 06,57, 06,61, 06,65, 06,69, 06,73
.db     12,25, 12,29, 12,33, 12,37, 12,41, 12,45, 12,49, 12,53, 12,57, 12,61, 12,65, 12,69, 12,73
#endif

hangTable:
#ifndef TI86
.dw     sprHang1,79*256+19,sprHang1,79*256+19
.dw     sprHang2_1,76*256+12,sprHang2_2,84*256+12
.dw     sprHang3,79*256+9,sprHang3,79*256+9
.dw     sprHang4_1,68*256+7,sprHang4_2,68*256+15
.dw     sprHang5_1,71*256+6,sprHang5_2,79*256+6
#else
.dw     sprHang1,95*256+19,sprHang1,95*256+19
.dw     sprHang2_1,92*256+12,sprHang2_2,100*256+12
.dw     sprHang3,95*256+9,sprHang3,95*256+9
.dw     sprHang4_1,84*256+7,sprHang4_2,84*256+15
.dw     sprHang5_1,87*256+6,sprHang5_2,95*256+6
#endif

.end
